@@ -8,17 +8,13 @@ from threading import _RLock,_Event,_Condition,_Semaphore,_BoundedSemaphore, \
 __all__ = ["active_count","activeCount","Condition","current_thread",
            "currentThread","enumerate","Event","local","Lock","RLock",
            "Semaphore","BoundedSemaphore","Thread","Timer","setprofile",
-           "settrace","stack_size"]
+           "settrace","stack_size","CPUSet","system_affinity",
+           "process_affinity"]
            
 
 
-#  Expose the actual class objects, rather than the strange function-based
-#  wrappers that threading wants to stick you with.
-Timer = _Timer
-
-
 class _ContextManagerMixin(object):
-    """Simple mixin providing __enter__ and __exit__."""
+    """Simple mixin mapping __enter__/__exit__ to acquire/release."""
 
     def __enter__(self):
         self.acquire()
@@ -41,11 +37,22 @@ class Lock(_ContextManagerMixin):
         super(Lock,self).__init__()
 
     def acquire(self,blocking=True,timeout=None):
+        """Attempt to acquire this lock.
+
+        If the optional argument "blocking" is True and "timeout" is None,
+        this methods blocks until is successfully acquires the lock.  If
+        "blocking" is False, it returns immediately if the lock could not
+        be acquired.  Otherwise, it blocks for at most "timeout" seconds
+        trying to acquire the lock.
+
+        In all cases, this methods returns True if the lock was successfully
+        acquired and False otherwise.
+        """
         if timeout is None:
             return self.__lock.acquire(blocking)
         else:
             #  Simulated timeout using progressively longer sleeps.
-            #  This is the same timeout scheme used in the standard Condition
+            #  This is the same timeout scheme used in the stdlib Condition
             #  class.  If there's lots of contention on the lock then there's
             #  a good chance you won't get it; but then again, Python doesn't
             #  guarantee fairness anyway.  We hope that platform-specific
@@ -61,10 +68,10 @@ class Lock(_ContextManagerMixin):
             return True
              
     def release(self):
+        """Release this lock."""
         self.__lock.release()
 
 
-
 class RLock(_ContextManagerMixin,_RLock):
     """Re-implemented RLock object.
 
@@ -226,10 +233,19 @@ class Event(object):
             return self.__cond.wait(timeout)
 
 
+class Timer(_Timer):
+    """Re-implemented Timer class.
+
+    Actually there's nothing new here, it just exposes the Timer class from
+    the stdlib as a normal class in case you want to extend it.
+    """
+    pass
+
+
 class Thread(Thread):
     """Extended Thread class.
 
-    This is a subclass of the standard python Thread, which adds support
+    This is a subclass of the standard python Thread class, which adds support
     for the following new features:
 
         * a "priority" attribute, through which you can set the (advisory)
@@ -247,9 +263,13 @@ class Thread(Thread):
 
     """
 
+    _ConditionClass = None
+
     def __init__(self,group=None,target=None,name=None,args=(),kwargs={},
                  daemon=None,priority=None,affinity=None):
         super(Thread,self).__init__(None,target,name,args,kwargs)
+        if self._ConditionClass is not None:
+            self.__block = self._ConditionClass()
         self.__ident = None
         if daemon is not None:
             self.daemon = daemon
@@ -258,11 +278,11 @@ class Thread(Thread):
         else:
             self.group = group
         if priority is not None:
-            self._set_priority(priority)
+            self.priority = priority
         else:
             self.__priority = None
         if affinity is not None:
-            self._set_affinity(affinity)
+            self.affinity = affinity
         else:
             self.__affinity = None
        
@@ -349,12 +369,85 @@ class Thread(Thread):
         if not 0 <= priority <= 1:
             raise ValueError("priority must be between 0 and 1")
         self.__priority = priority
+        if self.is_alive():
+            self.priority = priority
+        return priority
     priority = property(_get_priority,_set_priority)
+    def _set_priority(self,priority):
+        return priority
 
     def _get_affinity(self):
         return self.__affinity
     def _set_affinity(self,affinity):
+        if not isinstance(affinity,CPUSet):
+            affinity = CPUSet(affinity)
         self.__affinity = affinity
+        if self.is_alive():
+            self.affinity = affinity
+        return affinity
     affinity = property(_get_affinity,_set_affinity)
+    def _set_affinity(self,affinity):
+        return affinity
+
+
+#  Utilities for handling CPU affinity
+
+class CPUSet(set):
+    """Object representing a set of CPUs on which a thread is to run.
+
+    This is a python-level representation of the concept of a "CPU mask" as
+    used in various thread-affinity libraries.  Each CPU in the system is
+    represented by an integer, with the first being CPU zero.
+    """
+
+    def __init__(self,set_or_mask=None):
+        super(CPUSet,self).__init__()
+        if set_or_mask is not None:
+            if isinstance(set_or_mask,basestring):
+               for i in xrange(len(set_or_mask)):
+                   if set_or_mask[i] != "0":
+                       self.add(i)
+            elif isinstance(set_or_mask,int):
+                # TODO: do this calculation using shifts
+                cpu = 0
+                cur_mask = set_or_mask
+                while cur_mask:
+                    cpu_mask = 2**cpu
+                    if cpu_mask == cur_mask & cpu_mask:
+                         self.add(cpu)
+                    cur_mask ^= cpu_mask
+                    cpu += 1
+            else:
+                for i in set_or_mask:
+                    self.add(i)
+
+    def add(self,cpu):
+        return super(CPUSet,self).add(int(cpu))
+
+    def to_bitmask(self):
+        # TODO: do this calculation using shifts
+        bitmask = 0
+        for cpu in self:
+            bitmask |= 2**cpu
+        return bitmask
+
+
+def system_affinity():
+    """Get the set of CPUs available on this system."""
+    return CPUMask((0,))
+
+
+def process_affinity(affinity=None):
+    """Get or set the CPU affinity set for the current process.
+
+    This will affect all future threads spawned by this process.  It is
+    implementation-defined whether it will also affect previously-spawned
+    threads.
+    """
+    if affinity is not None:
+        affinity = CPUSet(affinity)
+        if affinity != system_affinity():
+            raise ValueError("unknown cpus: %s" % affinity)
+    return system_affinity()
 
 
